using System;
using System.Net;
using System.Runtime.CompilerServices;
using NUnit.Framework;

using Smdn.Net.Pop3.Protocol;

namespace Smdn.Net.Pop3.Client.Session {
  internal static class TestCases {
    internal static void WithConnectedSession(Action<PopSession> action,
                                              [CallerMemberName] string callerMemberName = "",
                                              [CallerFilePath] string sourceFilePath = "",
                                              [CallerLineNumber] int sourceLineNumber = 0)
    {
      WithConnectedSession((session, server) => action(session), callerMemberName, sourceFilePath, sourceLineNumber);
    }

    internal static void WithConnectedSession(Action<PopSession, PopPseudoServer> action,
                                              [CallerMemberName] string callerMemberName = "",
                                              [CallerFilePath] string sourceFilePath = "",
                                              [CallerLineNumber] int sourceLineNumber = 0)
    {
      PopPseudoServer.Start((server) => {
        using (var session = new PopSession(server.Host, server.Port)) {
          action(session, server);
        }
      }, callerMemberName, sourceFilePath, sourceLineNumber);
    }

    internal static void WithLoggedInSession(Action<PopSession, PopPseudoServer> action,
                                             [CallerMemberName] string callerMemberName = "",
                                             [CallerFilePath] string sourceFilePath = "",
                                             [CallerLineNumber] int sourceLineNumber = 0)
    {
      WithLoggedInSession(null, action, callerMemberName, sourceFilePath, sourceLineNumber);
    }

    internal static void WithLoggedInSession(PopCapability[] capabilities, Action<PopSession, PopPseudoServer> action,
                                             [CallerMemberName] string callerMemberName = "",
                                             [CallerFilePath] string sourceFilePath = "",
                                             [CallerLineNumber] int sourceLineNumber = 0)
    {
      WithConnectedSession((session, server) => {
        if (capabilities != null && 0 < capabilities.Length) {
          var resp = "+OK Capability list follows\r\n" +
                     string.Join("\r\n", (new PopCapabilitySet(capabilities)).ToStringArray()) +
                     "\r\n.\r\n";

          server.EnqueueResponse("CAPA\r\n",
                                 resp);

          Assert.IsTrue((bool)session.Capa());

          server.DequeueRequest(); //CAPA
        }

        server.EnqueueResponse("USER popuser\r\n",
                               "+OK\r\n");
        server.EnqueueResponse("PASS password\r\n",
                               "+OK\r\n");

        var credential = new NetworkCredential("popuser",
                                               "password",
                                               server.Host);

        Assert.IsTrue((bool)session.Login(credential));

        server.DequeueRequest(); // USER
        server.DequeueRequest(); // PASS

        Assert.AreEqual(PopSessionState.Transaction, session.State);

        session.HandlesIncapableAsException = true;

        action(session, server);
      }, callerMemberName, sourceFilePath, sourceLineNumber);
    }
  }
}
